/*
 * linux/arch/sh/edosk7707/setup.c
 *
 * Renesas EDOSK7705 Board Initialisation
 * Simon Dunn / Julian Back, MPC Data Limited, 2003.
 */

#include <linux/config.h>
#include <linux/init.h>
#include <linux/irq.h>

#include <linux/hdreg.h>
#include <linux/ide.h>
#include <asm/io.h>
#include <asm/delay.h>

/* Bus state controller registers for CS4 area */
#define BSC_CS4BCR    0xA4FD0010
#define BSC_CS4WCR    0xA4FD0030 

static void init_edosk7705_flash(void);


void heartbeat_edosk7705(void)
{
}

int __init setup_edosk7705(void)
{
        printk("Renesas EDOSK7705 port by MPC Data Limited\n");

	/* Setup CS4 area for access to Epson graphics card */
	ctrl_outl(0x424A0400, BSC_CS4BCR);
	ctrl_outl(0x00040981, BSC_CS4WCR);

    	return 0;
}

void init_edosk7705(void)
{
	/* This is the Ethernet interrupt */
	make_imask_irq(0x09);

	/* This is the IntelStrata Flash interrupt */
	make_imask_irq(0x0A);
}

static void edosk7705_dummy_IRQ(int irq, void * dev_id,  struct pt_regs * regs)
{
	/* The edosk7705 board has the IntelStrata flash STS
	   interrupt on 0x0A. We don't need this so ignore it here */
	
	udelay(1000);
}

static void init_edosk7705_flash(void)
{
	request_irq(0x0A, edosk7705_dummy_IRQ, SA_INTERRUPT, "flash", NULL);
}

static void exit_edosk7705_flash(void)
{
	free_irq(0x0A, NULL);	
}

module_init(init_edosk7705_flash);
module_exit(exit_edosk7705_flash);
