/*
 * linux/arch/sh/edosk7751r/setup.c
 *
 * Renesas EDOSK7751r Board Initialisation
 * Simon Dunn / Julian Back, MPC Data Limited, 2003.
 */

#include <linux/config.h>
#include <linux/init.h>
#include <linux/irq.h>

#include <linux/hdreg.h>
#include <linux/ide.h>
#include <asm/io.h>
#include <asm/delay.h>

static void init_edosk7751r_flash(void);


void heartbeat_edosk7751r(void)
{
}

int __init setup_edosk7751r(void)
{
        printk("Renesas EDOSK7751R port by MPC Data Limited\n");

    	return 0;
}

void init_edosk7751r(void)
{
    putDebugString("init_edosk7751r\n") ;   

	/* This is the Ethernet interrupt */
	make_imask_irq(0x09);

	/* This is the IntelStrata Flash interrupt */
	make_imask_irq(0x0A);
}

static void edosk7751r_dummy_IRQ(int irq, void * dev_id,  struct pt_regs * regs)
{
	/* The edosk7751r board has the IntelStrata flash STS
	   interrupt on 0x0A. We don't need this so ignore it here */
	
	udelay(1000);
}

static void init_edosk7751r_flash(void)
{
	request_irq(0x0A, edosk7751r_dummy_IRQ, SA_INTERRUPT, "flash", NULL);
}

static void exit_edosk7751r_flash(void)
{
	free_irq(0x0A, NULL);	
}

module_init(init_edosk7751r_flash);
module_exit(exit_edosk7751r_flash);

/********************************************************************************/

// ************************************************************
// DEBUG FUNCTIONS
//
#define SCIF_BASE 0xFFE80000

#define SCSMR  (volatile unsigned short *)(SCIF_BASE + 0x00)
#define SCBRR  (SCIF_BASE + 0x04)
#define SCSCR  (volatile unsigned short *)(SCIF_BASE + 0x08)
#define SC_TDR (volatile unsigned char *)(SCIF_BASE + 0x0C)
#define SC_SR  (volatile unsigned short *)(SCIF_BASE + 0x10)
#define SC_RDR (SCIF_BASE + 0x14)
#define SCFDR  (SCIF_BASE + 0x1C)
#define SCSPTR (SCIF_BASE + 0x20)
#define SCLSR  (SCIF_BASE + 0x24)

#define SCI_TD_E  0x0020
#define SCI_TDRE_CLEAR		0x00df

static inline int 
putDebugCharReady (void)
{
  unsigned short status;

  status = *SC_SR;
  return (status & SCI_TD_E);
}

void
putDebugChar (char ch)
{
  while (!putDebugCharReady())
    ;

  *SC_TDR = ch;
  *SC_SR;
  *SC_SR = SCI_TDRE_CLEAR;
}

void
putDebugString (char *str)
{
  char *p;

  for (p = str; *p; p++)
    {
      if (*p == '\n')
	putDebugChar ('\r');
      putDebugChar (*p);
    }
}
