/* 
 * linux/arch/sh/edosk7760/setup.c
 *
 * Copyright (C) 2000  Kazumoto Kojima
 *
 *
 * Modified for EDOSK7760 by
 * Richard Bister
 */

#include <linux/config.h>
#include <linux/init.h>
#include <linux/irq.h>

#include <linux/hdreg.h>
#include <linux/ide.h>
#include <asm/io.h>
#include <asm/delay.h>

#define SCIF_BASE 0xFE620000 /* Used for low-level debug output */

#define SCSMR  (volatile unsigned short *)(SCIF_BASE + 0x00)
#define SCBRR  (SCIF_BASE + 0x04)
#define SCSCR  (volatile unsigned short *)(SCIF_BASE + 0x08)
#define SC_TDR (volatile unsigned char *)(SCIF_BASE + 0x0C)
#define SC_SR  (volatile unsigned short *)(SCIF_BASE + 0x10)
#define SC_RDR (SCIF_BASE + 0x14)
#define SCTFDR (SCIF_BASE + 0x1C)
#define SCRFDR (SCIF_BASE + 0x20)
#define SCSPTR (SCIF_BASE + 0x24)
#define SCLSR  (SCIF_BASE + 0x28)
#define SCI_TD_E  0x0020
#define SCI_TDRE_CLEAR		0x00df

/* Bus state controller registers for CS4 area */
#define BSC_CS4BCR    0xA4FD0010
#define BSC_CS4WCR    0xA4FD0030 


/*
 * Initialize the board
 */
void __init setup_edosk7760(void)
{

	/* Setup CS4 area for access to Epson graphics card */
	ctrl_outl(0x424A0400, BSC_CS4BCR);
	ctrl_outl(0x00040981, BSC_CS4WCR);

	// TODO: Add any setup code here
	
}

void __init init_edosk7760(void)
{
	/* This is the Ethernet interrupt */
	make_imask_irq(0x09);

	/* This is the IntelStrata Flash interrupt */
	make_imask_irq(0x0A);
	
	/* Setup scif2 interrupts */
	sh7760_make_intc2_irq(SCIF2_ERI_IRQ, INTC2_BASE0, SCIF2_INTPRI_GROUP,
	 	SCIF2_ERI_INTMSK_POS, SCIF2_PRIORITY, SCIF2_INTMSK_OFFSET,
		SCIF2_INTPRI_OFFSET);
	
	sh7760_make_intc2_irq(SCIF2_RXI_IRQ, INTC2_BASE0, SCIF2_INTPRI_GROUP,
	 	SCIF2_RXI_INTMSK_POS, SCIF2_PRIORITY, SCIF2_INTMSK_OFFSET,
		SCIF2_INTPRI_OFFSET);
		
	sh7760_make_intc2_irq(SCIF2_BRI_IRQ, INTC2_BASE0, SCIF2_INTPRI_GROUP,
	 	SCIF2_BRI_INTMSK_POS, SCIF2_PRIORITY, SCIF2_INTMSK_OFFSET,
		SCIF2_INTPRI_OFFSET);	
		
	sh7760_make_intc2_irq(SCIF2_TXI_IRQ, INTC2_BASE0, SCIF2_INTPRI_GROUP,
	 	SCIF2_TXI_INTMSK_POS, SCIF2_PRIORITY, SCIF2_INTMSK_OFFSET,
		SCIF2_INTPRI_OFFSET);	

	sh7760_make_intc2_irq(USBHI_IRQ, INTC2_BASE0, 
			      USBHI_INTPRI_GROUP,
			      USBHI_INTMSK_POS, USBHI_PRIORITY, 
			      USBHI_INTMSK_OFFSET,
			      USBHI_INTPRI_OFFSET);	

	/* Setup dmabrg interrupts */
	sh7760_make_intc2_irq(DMABRGI0_IRQ, INTC2_BASE0, DMABRG_INTPRI_GROUP,
	 	DMABRGI0_INTMSK_POS, DMA_PRIORITY, DMABRG_INTMSK_OFFSET,
		DMABRG_INTPRI_OFFSET);

	sh7760_make_intc2_irq(DMABRGI1_IRQ, INTC2_BASE0, DMABRG_INTPRI_GROUP,
	 	DMABRGI1_INTMSK_POS, DMA_PRIORITY, DMABRG_INTMSK_OFFSET,
		DMABRG_INTPRI_OFFSET);
}

static void edosk7760_dummy_IRQ(int irq, void * dev_id,  struct pt_regs * regs)
{
	/* The edosk7760 board has the IntelStrata flash STS
	   interrupt on 0x0A. We don't need this so ignore it here */
	
  	udelay(1000); 
}

static void init_edosk7760_flash(void)
{
	request_irq(0x0A, edosk7760_dummy_IRQ, SA_INTERRUPT, "flash", NULL);
}

static void exit_edosk7760_flash(void)
{
	free_irq(0x0A, NULL);	
}

module_init(init_edosk7760_flash);
module_exit(exit_edosk7760_flash);





// ************************************************************
// DEBUG FUNCTIONS
//
static inline int 
putDebugCharReady (void)
{
  unsigned short status;

  status = *SC_SR;
  return (status & SCI_TD_E);
}

void
putDebugChar (char ch)
{
  while (!putDebugCharReady())
    ;

  *SC_TDR = ch;
  *SC_SR;
  *SC_SR = SCI_TDRE_CLEAR;
}

void
putDebugString (char *str)
{
  char *p;

  for (p = str; *p; p++)
    {
      if (*p == '\n')
	putDebugChar ('\r');
      putDebugChar (*p);
    }
}


