/*
 * linux/arch/sh/hico_sh3st/io.c
 *
 * Based largely on io_se.c.
 *
 * I/O routines for Emtrion HiCO.SH3ST board.
 *
 */

#include <linux/kernel.h>
#include <linux/types.h>
#include <asm/io.h>
#include <asm/io_hico_sh3st.h>
#include <asm/addrspace.h>

#include <linux/pci.h>

/* The Ethernet controller lives in Area 2 : */
#define SMC_IOADDR	0xA8000000

#define maybebadio(name,port) \
  printk("bad PC-like io %s for port 0x%lx at 0x%08x\n", \
	 #name, (port), (__u32) __builtin_return_address(0))

/* Map the Ethernet addresses as if it is at 0x300 - 0x320 */

static inline unsigned long __port2addr(unsigned long port)
{
     if (port >= 0x300 && port < 0x320)
     {
	  return SMC_IOADDR + port;
     }
     
     maybebadio(__port2addr, port);
     return port;
}

static inline void delay(void)
{
	ctrl_inw(0xa0000000);
}

/* Trying to read / write bytes on odd-byte boundaries to the Ethernet
 * registers causes problems. So we bit-shift the value and read / write
 * in 2 byte chunks. Setting the low byte to 0 does not cause problems
 * now as odd byte writes are only made on the bit mask / interrupt
 * register. This may not be the case in future Mar-2003 SJD
 */
unsigned char sh_hico_sh3st_inb(unsigned long port)
{
	if (port >= 0x300 && port < 0x320 && port & 0x01) {
		return (volatile unsigned char)(sh_hico_sh3st_inw(port -1) >> 8);
	}
	return *(volatile unsigned char *)__port2addr(port);
}

unsigned short sh_hico_sh3st_inw(unsigned long port)
{
	return *(volatile unsigned short *)__port2addr(port);
}

unsigned int sh_hico_sh3st_inl(unsigned long port)
{
	return *(volatile unsigned long *)port;
}

void sh_hico_sh3st_outb(unsigned char value, unsigned long port)
{
	if (port >= 0x300 && port < 0x320 && port & 0x01) {
		sh_hico_sh3st_outw(((unsigned short)value << 8), port -1);
		return;
	}
	*(volatile unsigned char *)__port2addr(port) = value;
}

void sh_hico_sh3st_outw(unsigned short value, unsigned long port)
{
	*(volatile unsigned short *)__port2addr(port) = value;
}

void sh_hico_sh3st_outl(unsigned int value, unsigned long port)
{
	*(volatile unsigned long *)port = value;
}

unsigned char sh_hico_sh3st_inb_p(unsigned long port)
{
	unsigned char v;

 	v = *(volatile unsigned char *)__port2addr(port);
        return v;
}

void sh_hico_sh3st_outb_p(unsigned char value, unsigned long port)
{
	*(volatile unsigned char *)__port2addr(port) = value;
}

void sh_hico_sh3st_insb(unsigned long port, void *addr, unsigned long count)
{
	unsigned char *p = addr;
	while (count--) *p++ = sh_hico_sh3st_inb(port);
}

void sh_hico_sh3st_insw(unsigned long port, void *addr, unsigned long count)
{
	unsigned short *p = (unsigned short*)addr;
	while (count--) *p++ = sh_hico_sh3st_inw(port);
}

void sh_hico_sh3st_insl(unsigned long port, void *addr, unsigned long count)
{
	unsigned long *p = (unsigned long*)addr;
	while (count--) *p++ = sh_hico_sh3st_inl(port);
}

void sh_hico_sh3st_outsb(unsigned long port, const void *addr, unsigned long count)
{
	unsigned char *p = (unsigned char*)addr;
	while (count--) sh_hico_sh3st_outb(*p++, port);
}

void sh_hico_sh3st_outsw(unsigned long port, const void *addr, unsigned long count)
{
	unsigned short *p = (unsigned short*)addr;
	while (count--) sh_hico_sh3st_outw(*p++, port);
}

void sh_hico_sh3st_outsl(unsigned long port, const void *addr, unsigned long count)
{
	unsigned long *p = (unsigned long*)addr;
	while (count--) sh_hico_sh3st_outl(*p++, port);
}

unsigned char sh_hico_sh3st_readb(unsigned long addr)
{
	return *(volatile unsigned char*)addr;
}

unsigned short sh_hico_sh3st_readw(unsigned long addr)
{
	return *(volatile unsigned short*)addr;
}

unsigned int sh_hico_sh3st_readl(unsigned long addr)
{
	return *(volatile unsigned long*)addr;
}

void sh_hico_sh3st_writeb(unsigned char b, unsigned long addr)
{
	*(volatile unsigned char*)addr = b;
}

void sh_hico_sh3st_writew(unsigned short b, unsigned long addr)
{
	*(volatile unsigned short*)addr = b;
}

void sh_hico_sh3st_writel(unsigned int b, unsigned long addr)
{
	*(volatile unsigned long*)addr = b;
}

unsigned long sh_hico_sh3st_isa_port2addr(unsigned long offset)
{
     return __port2addr(offset);
}
