/*
 * linux/arch/sh/hico_sh3st/led.c
 *
 * May be copied or modified under the terms of the GNU General Public
 * License.  See linux/COPYING for more information.
 *
 * This file contains HICO.SH3ST and compatible code.
 */

#include <linux/config.h>
#include <asm/io.h>
#include "hico_sh3st.h"


/*
 * On the HiCO.SH3ST there are two LED's one Green and one Yellow. 
 * They are connected :
 *   Yellow - PTJ6
 *   Green  - PTJ7
 *   (Write a 0 to light the LED)
 *
 * Note that Port J is setup for output in setup.c
 */
static void mach_led(int position, int value)
{
	unsigned char pjdr_state;


	/* Write the port control register to set outputs. */
	pjdr_state = ctrl_inb(PJDR);
	if (value) {
                pjdr_state &= ~(PJ6DT);
                pjdr_state |= PJ7DT;
	} else {
	        pjdr_state &= ~(PJ7DT);
		pjdr_state |= PJ6DT;
	}
	ctrl_outb(pjdr_state, PJDR);
}



#ifdef CONFIG_HEARTBEAT

#include <linux/sched.h>

/* acts like an actual heart beat -- ie thump-thump-pause... */
void heartbeat_hico_sh3st(void)
{
	static unsigned cnt = 0, period = 0, dist = 0;

	if (cnt == 0 || cnt == dist)
		mach_led( -1, 1);
	else if (cnt == 7 || cnt == dist+7)
		mach_led( -1, 0);

	if (++cnt > period) {
		cnt = 0;
		/* The hyperbolic function below modifies the heartbeat period
		 * length in dependency of the current (5min) load. It goes
		 * through the points f(0)=126, f(1)=86, f(5)=51,
		 * f(inf)->30. */
		period = ((672<<FSHIFT)/(5*avenrun[0]+(7<<FSHIFT))) + 30;
		dist = period / 4;
	}
}
#endif
