/* FLCTL.H - Contains definitions of the SH4 7760 Flash Controller */

#ifndef FLCTL_H
#define FLCTL_H

// Base address
#define FLCTL_BASE      (0xFE440000)

// Register offsets
#define FLCMNCR_OFFSET  (0x00)
#define FLCMDCR_OFFSET  (0x04)
#define FLCMCDR_OFFSET  (0x08)
#define FLADR_OFFSET    (0x0C)
#define FLDATAR_OFFSET  (0x10)
#define FLDTCNTR_OFFSET (0x14)
#define FLINTDMACR_OFFSET (0x18)
#define FLBSYTMR_OFFSET (0x1C)
#define FLBSYCNT_OFFSET (0x20)
#define FLDTFIFO_OFFSET (0x24)
#define FLECFIFO_OFFSET (0x28)
#define FLTRCR_OFFSET   (0x2C)

#define REG_ACCESS(reg)   (*((volatile unsigned int   *) (FLCTL_BASE + reg)))
#define REG_ACCESS8(reg)  (*((volatile unsigned char  *) (FLCTL_BASE + reg)))

// Registers
#define FLCMNCR     REG_ACCESS(FLCMNCR_OFFSET)
#define FLCMDCR     REG_ACCESS(FLCMDCR_OFFSET)
#define FLCMCDR     REG_ACCESS(FLCMCDR_OFFSET)
#define FLADR       REG_ACCESS(FLADR_OFFSET)
#define FLDATAR     REG_ACCESS(FLDATAR_OFFSET)
#define FLDTCNTR    REG_ACCESS(FLDTCNTR_OFFSET)
#define FLINTDMACR  REG_ACCESS(FLINTDMACR_OFFSET)
#define FLBSYTMR    REG_ACCESS(FLBSYTMR_OFFSET)
#define FLBSYCNT    REG_ACCESS(FLBSYCNT_OFFSET)
#define FLDTFIFO    REG_ACCESS(FLDTFIFO_OFFSET)
#define FLECFIFO    REG_ACCESS(FLECFIFO_OFFSET)
#define FLTRCR      REG_ACCESS8(FLTRCR_OFFSET)

#define BIT(x)      (1 << (x))

// FLCMNCR bits
#define FCKSEL      BIT(15)
#define ECCPOS1     BIT(13)
#define ECCPOS0     BIT(12)
#define ACM1        BIT(11)
#define ACM0        BIT(10)
#define NANDWF      BIT(9)
#define CE0         BIT(3)
#define TYPESEL     BIT(0)

#define ECC_POS_0_7     (0x00000000)
#define ECC_POS_4_11    (0x00001000)
#define ECC_POS_8_15    (0x00002000)

#define ACM_COMMAND     (0x00000000)
#define ACM_SECTOR      (0x00004000)

// FLCMDCR
#define ADRMD       BIT(26)
#define CDSRC       BIT(25)
#define DOSR        BIT(24)
#define SELRW       BIT(21)
#define DOADR       BIT(20)
#define ADRCNT1     BIT(19)
#define ADRCNT0     BIT(18)
#define DOCMD2      BIT(17)
#define DOCMD1      BIT(16)

#define ADRCNT_1_BYTE   (0x00000000) 
#define ADRCNT_2_BYTE   (0x00040000)
#define ADRCNT_3_BYTE   (0x00080000)
#define ADRCNT_4_BYTE   (0x000C0000)

#define SCTCNT(x)   ((x) & 0xFFFF)

#define FLASH_READ  (0x00000000)
#define FLASH_WRITE (0x00200000)
// FLCMCDR
#define CMD_0(x)    (((x) & 0xFF) << 0)
#define CMD_1(x)    (((x) & 0xFF) << 8)

// FLADR
#define ADR_0(x)    (((x) & 0xFF) << 0)
#define ADR_1(x)    (((x) & 0xFF) << 8)
#define ADR_2(x)    (((x) & 0xFF) << 16)
#define ADR_3(x)    (((x) & 0xFF) << 24)

// FLDTCNTR
#define ECFLW(x)    (((x) & 0xFF000000) >> 24)
#define DTFLW(x)    (((x) & 0x00FF0000) >> 16)
#define DTCNT(x)    (((x) & 0x00000FFF) << 0)

// FLDATAR
// Write
#define DATAW_0(x)  (((x) & 0xFF) << 0)
#define DATAW_1(x)  (((x) & 0xFF) << 8)
#define DATAW_2(x)  (((x) & 0xFF) << 16)
#define DATAW_3(x)  (((x) & 0xFF) << 24)
// Read
#define DATAR_0(x)  (((x) & 0x000000FF) >> 0)
#define DATAR_1(x)  (((x) & 0x0000FF00) >> 8)
#define DATAR_2(x)  (((x) & 0x00FF0000) >> 16)
#define DATAR_3(x)  (((x) & 0xFF000000) >> 24)

// FLINTDMACR
#define ECERINTE    BIT(24)
#define FIFOTRG1    BIT(21)
#define FIFOTRG0    BIT(20)
#define AC1CLR      BIT(19)
#define AC0CLR      BIT(18)
#define DREQ1EN     BIT(17)
#define DREQ0EN     BIT(16)
#define DTOERB      BIT(15)
#define DTUERB      BIT(14)
#define ECOERB      BIT(13)
#define ECUERB      BIT(12)
#define ECERB       BIT(9)
#define STERB       BIT(8)
#define BTOERB      BIT(7)
#define TRREQF1     BIT(6)
#define TRREQF0     BIT(5)
#define STERINTE    BIT(4)
#define RBERINTE    BIT(3)
#define TEINTE      BIT(2)
#define TRINTE1     BIT(1)
#define TRINTE0     BIT(0)

#define FIFO_4      (0x00000000)
#define FIFO_16     (0x00100000)
#define FIFO_128    (0x00200000)

// FLBSYTMR
#define RBTMOUT(x)  ((x) & 0x000FFFFF)

// FLBSYCNT
#define STAT(x)     (((x) & 0xFF000000) >> 24)
#define RBTIMCNT(x) (((x) & 0x000FFFFF) >> 0)

// FLDTFIFO

// FLECFIFO

// FLTRCR
#define TREND       BIT(1)
#define TRSTART     BIT(0)

// Strata Flash
#define NOR_READ_ARRAY                0x00FF00FF
#define NOR_ERASE_SETUP               0x00200020
#define NOR_ERASE_CONFIRM             0x00D000D0
#define NOR_CMD_CLEAR_LOCK            0x00600060
#define NOR_CMD_WRITE	          0x00400040
#define NOR_PGM_SETUP                 CMD_WRITE
#define NOR_STATUS_READ               0x00700070
#define NOR_STATUS_CLEAR              0x00500050
#define NOR_CMD_READ_QUERY            0x00980098
#define NOR_CMD_WR_BUFF               0x00e800e8
#define NOR_CMD_CONFIRM               0x00D000D0
#define NOR_DEV_MAN_READ              0x00900090

#define NOR_STATUS_PROTECT            0x00020002
#define NOR_STATUS_VOLTAGE_ERR        0x00080008
#define NOR_STATUS_PGM_ERR            0x00100010
#define NOR_STATUS_ERASE_ERR          0x00200020
#define NOR_STATUS_BUSY               0x00800080

#define NOR_BASE                  0xa0000000

#define NOR_ACCESS(x) (*(volatile unsigned long *)(x))

// BSC
#define BSC_BCR2 0xff800004

// Test boot flash
#define BOOT_FLASH_IS_32BIT() (0xc000 == ((*(volatile unsigned short *)BSC_BCR2) & 0xc000))

#endif
