/* NANDCMDS.H - NAND flash command list */
/* Contains list of NAND flash commands that are used */

#ifndef NANDCMDS_H
#define NANDCMDS_H

/* Prepare chip buffer for data input for programming */
#define SERIAL_DATA_INPUT   (0x80)

/* Read mode 1 - start read at bytes 0-255 */
#define READ_MODE_1         (0x00)
/* Read mode 2 - start read at bytes 256-511 */
#define READ_MODE_2         (0x01)
/* Read mode 3 - start read at bytes 512-527 (OOB) */
#define READ_MODE_3         (0x50)

/* Reset - Reset chip */
#define RESET               (0xFF)

/* Auto Program - program buffer into array */
#define AUTO_PROGRAM        (0x10)
/* Auto Program Dummy - dummy program command (used for multi-block program */
#define AUTO_PROGRAM_DUMMY  (0x11)
/* Auto Program Multi - multi-block programming. Used with AUTO_PROGRAM_DUMMY */
#define AUTO_PROGRAM_MULTI  (0x15)

/* Auto Block Erase - erases a block */
#define AUTO_BLOCK_ERASE    (0x60)
/* Auto Block Erase confirm - sent to confirm an erase (after address) */
#define AUTO_BLOCK_ERASE_CONFIRM    (0xD0)

/* Read status register */
#define READ_STATUS         (0x70)
/* Read status register 2 - Not supported on some chips */
#define READ_STATUS_2       (0x71)

/* Read ID register */
#define READ_ID             (0x90)
/* Read second ID register - not supported on some chips */
#define READ_ID_2           (0x91)

/* Status register bits */
#define STATUS_PASSFAIL     (0x01)
#define STATUS_PASS         (0x00)
#define STATUS_FAIL         (0x01)

#define STATUS_READYBUSY    (0x40)
#define STATUS_READY        (0x40)
#define STATUS_BUSY         (0x00)

#define STATUS_WRITEPROTECT (0x80)
#define STATUS_PROTECT      (0x00)
#define STATUS_NOTPROTECTED (0x80)


#endif // NANDCMDS_H
