/* 
 * linux/arch/sh/jci_n40/setup.c
 *
 * Copyright (C) 2000  Kazumoto Kojima
 *
 *
 * Modified for JCI N40 by
 * Dmitry Popov
 */

#include <linux/config.h>
#include <linux/init.h>
#include <linux/irq.h>

#include <linux/hdreg.h>
#include <linux/ide.h>
#include <asm/io.h>
#include <asm/delay.h>

// BSC
#define BSC_BCR2 0xff800004

// Test boot flash
#define BOOT_FLASH_IS_32BIT() (0xc000 == ((*(volatile unsigned short *)BSC_BCR2) & 0xc000))

/* Serial console port suppression */
#define SERIAL_CONSOLE_SUPP_ADDR 0xfe2e0600
#define SERIAL_CONSOLE_SUPP_FLAG 0x70707573
#define SERIAL_CONSOLE_SUPPRESS() (m_serial_console_suppress)

/* <optimize engr="kweller"> Use static to store DUART base address </optimize> */
#define UART_BASE (BOOT_FLASH_IS_32BIT() ? 0xB8000008 : 0xA4000008)

#define FFTHR     0x00
#define FFLSR     0x05

#define TSTR    0xFFD80004 // Timer Start
#define TCR2    0xFFD80028 // Timer2 Ctrl.

/* Bus state controller registers for CS4 area */
//#define BSC_CS4BCR    0xA4FD0010
//#define BSC_CS4WCR    0xA4FD0030 

// GPIO Registers
#define PBCR    0xFE400004
#define PBDR    0xFE400044

static int m_serial_console_suppress;

/*
 * Initialize the board
 */
void __init setup_jci_n40(void)
{
	// TODO: Add any setup code here
        m_serial_console_suppress = (SERIAL_CONSOLE_SUPP_FLAG == *(unsigned long *)SERIAL_CONSOLE_SUPP_ADDR);
        /* <debug> quick test </debug> */
        putDebugString ("setup_jci_n40");

}

void __init init_jci_n40(void)
{
	/* Setup scif2 interrupts */
/*	sh7760_make_intc2_irq(SCIF2_ERI_IRQ, INTC2_BASE0, SCIF2_INTPRI_GROUP,
	 	SCIF2_ERI_INTMSK_POS, SCIF2_PRIORITY, SCIF2_INTMSK_OFFSET,
		SCIF2_INTPRI_OFFSET);
	
	sh7760_make_intc2_irq(SCIF2_RXI_IRQ, INTC2_BASE0, SCIF2_INTPRI_GROUP,
	 	SCIF2_RXI_INTMSK_POS, SCIF2_PRIORITY, SCIF2_INTMSK_OFFSET,
		SCIF2_INTPRI_OFFSET);
		
	sh7760_make_intc2_irq(SCIF2_BRI_IRQ, INTC2_BASE0, SCIF2_INTPRI_GROUP,
	 	SCIF2_BRI_INTMSK_POS, SCIF2_PRIORITY, SCIF2_INTMSK_OFFSET,
		SCIF2_INTPRI_OFFSET);	
		
	sh7760_make_intc2_irq(SCIF2_TXI_IRQ, INTC2_BASE0, SCIF2_INTPRI_GROUP,
	 	SCIF2_TXI_INTMSK_POS, SCIF2_PRIORITY, SCIF2_INTMSK_OFFSET,
		SCIF2_INTPRI_OFFSET);	

	sh7760_make_intc2_irq(USBHI_IRQ, INTC2_BASE0, 
			      USBHI_INTPRI_GROUP,
			      USBHI_INTMSK_POS, USBHI_PRIORITY, 
			      USBHI_INTMSK_OFFSET,
			      USBHI_INTPRI_OFFSET);
*/

	/* Setup dmabrg interrupts */
	sh7760_make_intc2_irq(DMABRGI0_IRQ, INTC2_BASE0, DMABRG_INTPRI_GROUP,
	 	DMABRGI0_INTMSK_POS, DMA_PRIORITY, DMABRG_INTMSK_OFFSET,
		DMABRG_INTPRI_OFFSET);

	sh7760_make_intc2_irq(DMABRGI1_IRQ, INTC2_BASE0, DMABRG_INTPRI_GROUP,
	 	DMABRGI1_INTMSK_POS, DMA_PRIORITY, DMABRG_INTMSK_OFFSET,
		DMABRG_INTPRI_OFFSET);

    make_imask_irq(0x12); // Timer2 interrupt
    /* Disable Timer */
    ctrl_outb(ctrl_inb(TSTR) & 0x3, TSTR); // disable timer
    ctrl_outw(0, TCR2); // deinitialize timer2

    ctrl_outw(ctrl_inw(0xFFD00000) | 0x0080, 0xFFD00000); // Set IRL as independent
    ctrl_outw(0x1100, 0xFFD00010); // Enable RS232 and Ethernet interrupts only
    make_imask_irq(0x05); // experimentally - RS232
    make_imask_irq(0x02); // experimentally - Ethernet

    // Initialize GPIOs
    ctrl_outw((ctrl_inw(PBCR) & 0xF3FC), PBCR); // PB5 set as GPIO out
    ctrl_outw((ctrl_inw(PBCR) | 0x0400), PBCR); // PB5 set as GPIO out
}

static void jci_n40_dummy_IRQ(int irq, void * dev_id,  struct pt_regs * regs)
{
	
  	udelay(1000); 
}

static void init_jci_n40_flash(void)
{
	request_irq(0x0A, jci_n40_dummy_IRQ, SA_INTERRUPT, "flash", NULL);
}

static void exit_jci_n40_flash(void)
{
	free_irq(0x0A, NULL);	
}

//module_init(init_jci_n40_flash);
//module_exit(exit_jci_n40_flash);





// ************************************************************
// DEBUG FUNCTIONS
//
static inline int 
putDebugCharReady (void)
{
	return ((*(volatile u8 *)(UART_BASE + FFLSR) & 0x20) != 0);
}

void
putDebugChar (char ch)
{
  if (!SERIAL_CONSOLE_SUPPRESS())
  {
	while (!putDebugCharReady()) {}

	*(volatile u8 *)(UART_BASE + FFTHR) = ((u8)ch);
  }
}

void
putDebugString (char *str)
{
  char *p;

  for (p = str; *p; p++)
    {
      if (*p == '\n')
	putDebugChar ('\r');
      putDebugChar (*p);
    }
}


