/* SH4GPIO.H - SH4 GPIO Routines */

#ifndef SH4GPIO_H
#define SH4GPIO_H

// Base
#define GPIO_BASE   (0xFE400000)
// Offsets
#define INPUPA      (0x28)
#define DMAPCR      (0x2C)
#define SCIHZR      (0x30)
#define IPSELR      (0x34)
// Pullup regs
#define PAPUPR      (0x80)
#define PBPUPR      (0x84)
#define PCPUPR      (0x88)
#define PDPUPR      (0x8C)
#define PEPUPR      (0x90)
#define PFPUPR      (0x94)
#define PGPUPR      (0x98)
#define PHPUPR      (0x9C)
#define PJPUPR      (0xA0)
#define PKPUPR      (0xA4)
#define MDPUPR      (0xA8)
#define MODSELR     (0xAc)

// Control Regs
#define PACR        (0x00)
#define PBCR        (0x04)
#define PCCR        (0x08)
#define PDCR        (0x0C)
#define PECR        (0x10)
#define PFCR        (0x14)
#define PGCR        (0x18)
#define PHCR        (0x1C)
#define PJCR        (0x20)
#define PKCR        (0x24)

// Data Regs
#define PADR        (0x40)
#define PBDR        (0x44)
//#define PCDR        (0x48)
#define PDDR        (0x4C)
//#define PEDR        (0x50)
#define PFDR        (0x54)
#define PGDR        (0x58)
#define PHDR        (0x5C)
#define PJDR        (0x60)
#define PKDR        (0x64)

// Interrupt
#define GPIOIC      (0x00400048)

// Control reg configuration
#define CTRL_PERIPH (0x0)
#define CTRL_OUTPUT (0x1)
#define CTRL_INPUTN (0x2) // No pullup
#define CTRL_INPUTP (0x3) // Pullup

#define SET_CTRL_REG(ctrl_reg, port, value) (*((u16*) GPIO_BASE + ctrl_reg) = (*((u16*) GPIO_BASE + ctrl_reg) & ~(CTRL_INPUTP << (2*port))) | (value << (2*port)))

// Ports
// A
#define PA7         (7)
#define PA6         (6)
#define PA5         (5)
#define PA4         (4)
#define PA3         (3)
#define PA2         (2)
// B
#define PB7         (7)
#define PB6         (6)
#define PB5         (5)
#define PB4         (4)
#define PB3         (3)
#define PB2         (2)
#define PB1         (1)
// C
#define PC7         (7)
#define PC6         (6)
#define PC5         (5)
#define PC4         (4)
#define PC3         (3)
#define PC2         (2)
#define PC1         (1)
#define PC0         (0)
// D
#define PD7         (7)
#define PD6         (6)
#define PD5         (5)
#define PD4         (4)
#define PD3         (3)
#define PD2         (2)
#define PD1         (1)
#define PD0         (0)
// E
#define PE7         (7)
#define PE6         (6)
#define PE5         (5)
#define PE4         (4)
#define PE3         (3)
#define PE2         (2)
#define PE1         (1)
#define PE0         (0)
// F
#define PF3         (3)
#define PF2         (2)
#define PF1         (1)
#define PF0         (0)
// G
#define PG7         (7)
#define PG6         (6)
#define PG5         (5)
#define PG4         (4)
#define PG3         (3)
#define PG2         (2)
#define PG1         (1)
#define PG0         (0)
// H
#define PH7         (7)
#define PH6         (6)
#define PH5         (5)
#define PH4         (4)
#define PH3         (3)
#define PH2         (2)
#define PH1         (1)
#define PH0         (0)
// J
#define PJ7         (7)
#define PJ6         (6)
#define PJ5         (5)
#define PJ4         (4)
#define PJ3         (3)
#define PJ2         (2)
#define PJ1         (1)
// K
#define PK7         (7)
#define PK6         (6)
#define PK5         (5)
#define PK4         (4)
#define PK3         (3)
#define PK2         (2)
#define PK1         (1)

#endif // SH4GPIO_H
