/* $Id: cache-sh3.c,v 1.1.1.1 2004/03/16 23:54:10 dpopov Exp $
 *
 *  linux/arch/sh/mm/cache-sh3.c
 *
 * Copyright (C) 1999, 2000  Niibe Yutaka
 *
 */

#include <linux/init.h>
#include <linux/mman.h>
#include <linux/mm.h>
#include <linux/threads.h>
#include <asm/addrspace.h>
#include <asm/page.h>
#include <asm/pgtable.h>
#include <asm/processor.h>
#include <asm/cache.h>
#include <asm/io.h>
#include <asm/uaccess.h>
#include <asm/pgalloc.h>
#include <asm/mmu_context.h>


#define CCR		0xffffffec	/* Address of Cache Control Register */

#if defined(CONFIG_CPU_SUBTYPE_SH7705)
#define CCR1		CCR
#define CCR2		0xA40000B0
#define CCR3		0xA40000B4

#define CACHE_SIZE_16KB 0x00010000
#define CACHE_SIZE_32KB 0x00020000
#endif

#define CCR_CACHE_CE	0x01	/* Cache Enable */
#define CCR_CACHE_WT	0x02	/* Write-Through (for P0,U0,P3) (else writeback) */
#define CCR_CACHE_CB	0x04	/* Write-Back (for P1) (else writethrough) */
#define CCR_CACHE_CF	0x08	/* Cache Flush */
#define CCR_CACHE_RA	0x20	/* RAM mode */

#define CCR_CACHE_VAL	(CCR_CACHE_CB|CCR_CACHE_CE)	/* 8k-byte cache, P1-wb, enable */
#define CCR_CACHE_INIT	(CCR_CACHE_CF|CCR_CACHE_VAL)	/* 8k-byte cache, CF, P1-wb, enable */

#define CACHE_OC_ADDRESS_ARRAY 0xf0000000
#define CACHE_VALID	  1
#define CACHE_UPDATED	  2
#define CACHE_PHYSADDR_MASK 0x1ffffc00

#define CACHE_LRU 0x2f0

/* Data at BSS is cleared after setting this variable.
   So, we Should not placed this variable at BSS section.
   Initialize this, it is placed at data section. */
struct _cache_system_info cache_system_info = {0,};

#define CACHE_OC_WAY_SHIFT	(cache_system_info.way_shift)
#define CACHE_OC_ENTRY_SHIFT    4
#define CACHE_OC_ENTRY_MASK	(cache_system_info.entry_mask)
#define CACHE_OC_NUM_ENTRIES	(cache_system_info.num_entries)
#define CACHE_OC_NUM_WAYS	4
#define CACHE_OC_ASSOC_BIT    (1<<3)



/*
 * Write back all the cache.
 *
 * For SH-4, we only need to flush (write back) Operand Cache,
 * as Instruction Cache doesn't have "updated" data.
 *
 * Assumes that this is called in interrupt disabled context, and P2.
 * Shuld be INLINE function.
 */
static inline void cache_wback_all(void)
{
	unsigned long addr, data, i, j = 0;

	for (i=0; i<CACHE_OC_NUM_ENTRIES; i++) {
		for (j=0; j<CACHE_OC_NUM_WAYS; j++) {
			addr = CACHE_OC_ADDRESS_ARRAY|(j<<CACHE_OC_WAY_SHIFT)|
				(i<<CACHE_OC_ENTRY_SHIFT);
			data = ctrl_inl(addr);
			if ((data & (CACHE_UPDATED|CACHE_VALID))
			    == (CACHE_UPDATED|CACHE_VALID))
				ctrl_outl(data & ~(CACHE_VALID |CACHE_UPDATED), addr);
		}
	}
}


static inline void cache_purge_all(void)
{
	unsigned long addr, data, i, j = 0;

	for (i=0; i<CACHE_OC_NUM_ENTRIES; i++) {
		for (j=0; j<CACHE_OC_NUM_WAYS; j++) {
			addr = CACHE_OC_ADDRESS_ARRAY|(j<<CACHE_OC_WAY_SHIFT)|
				(i<<CACHE_OC_ENTRY_SHIFT);
			data = ctrl_inl(addr);
			if (data & CACHE_VALID) {
				if (data & CACHE_UPDATED) {
					ctrl_outl(data & ~CACHE_UPDATED, addr);
				}
				ctrl_outl(data & ~CACHE_VALID, addr);
			}
		}
	}
}


static void __init
detect_cpu_and_cache_system(void)
{
	unsigned long addr0, addr1, data0, data1, data2, data3;

#if defined(CONFIG_CPU_SUBTYPE_SH7705) 
	/* Set to 16KB initially which should be the default values */
	cpu_data->type = CPU_SH7705;

	cache_system_info.way_shift = 12;
	cache_system_info.entry_mask = 0xFF0;
	cache_system_info.num_entries = 256;
	return;
#endif

	jump_to_P2();
	/*
	 * Check if the entry shadows or not.
	 * When shadowed, it's 128-entry system.
	 * Otherwise, it's 256-entry system.
	 */
	addr0 = CACHE_OC_ADDRESS_ARRAY + (3 << 12);
	addr1 = CACHE_OC_ADDRESS_ARRAY + (1 << 12);

	/* First, write back & invalidate */
	data0  = ctrl_inl(addr0);
	ctrl_outl(data0&~(CACHE_VALID|CACHE_UPDATED), addr0);
	data1  = ctrl_inl(addr1);
	ctrl_outl(data1&~(CACHE_VALID|CACHE_UPDATED), addr1);

	/* Next, check if there's shadow or not */
	data0 = ctrl_inl(addr0);
	data0 ^= CACHE_VALID;
	ctrl_outl(data0, addr0);
	data1 = ctrl_inl(addr1);
	data2 = data1 ^ CACHE_VALID;
	ctrl_outl(data2, addr1);
	data3 = ctrl_inl(addr0);

	/* Lastly, invaliate them. */
	ctrl_outl(data0&~CACHE_VALID, addr0);
	ctrl_outl(data2&~CACHE_VALID, addr1);
	back_to_P1();

	if (data0 == data1 && data2 == data3) {	/* Shadow */
		cache_system_info.way_shift = 11;
		cache_system_info.entry_mask = 0x7f0;
		cache_system_info.num_entries = 128;
		cpu_data->type = CPU_SH7708;
	} else {				/* 7709A or 7729  */
		cache_system_info.way_shift = 12;
		cache_system_info.entry_mask = 0xff0;
		cache_system_info.num_entries = 256;
		cpu_data->type = CPU_SH7729;
	}
}

void __init cache_init(void)
{
	unsigned long ccr;
#if defined(CONFIG_CPU_SUBTYPE_SH7705) && defined(CONFIG_SH_CACHE_32KB)
	unsigned long j = 0, i, addr;
#endif


	detect_cpu_and_cache_system();
	jump_to_P2();
#if defined(CONFIG_CPU_SUBTYPE_SH7705) && defined(CONFIG_SH_CACHE_32KB)
	ccr = ctrl_inl(CCR1);
	ctrl_outl(0, CCR2);


	cache_system_info.way_shift = 13;
	cache_system_info.entry_mask = 0x1FF0;
	cache_system_info.num_entries = 512;

	/* Invalidate every entry in the cache */
	for (j = 0; j < 4; j++) {
		for (i = 0; i < 256; i++) {
			addr = 	0xF0000000 |
				(j << 12) |
				(i << 4);

			ctrl_outl((ctrl_inl(addr) & (~CACHE_VALID & ~CACHE_UPDATED)), addr);
		}
	}
	/* Reset the size of the cache in CCR3 */
	ctrl_outl(CACHE_SIZE_32KB, CCR3);

//	ctrl_outl(CCR_CACHE_INIT, CCR1);
	ctrl_outl((ccr | CCR_CACHE_CF), CCR1);

#endif
	ccr = ctrl_inl(CCR);
	if (ccr & CCR_CACHE_CE)
		/*
		 * XXX: Should check RA here.
		 * If RA was 1, we only need to flush the half of the caches.
		 */
		cache_wback_all();

	ctrl_outl(CCR_CACHE_INIT, CCR);

	back_to_P1();
}

/*
 * Write back the dirty D-caches, but not invalidate them.
 *
 * Is this really worth it, or should we just alias this routine
 * to __flush_purge_region too?
 *
 * START: Virtual Address (U0, P1, or P3)
 * SIZE: Size of the region.
 */

inline void __flush_wback_region(void *start, int size)
{
	unsigned long v, j;
	unsigned long begin, end;
	unsigned long flags;

	begin = (unsigned long)start & ~(L1_CACHE_BYTES-1);
	end = ((unsigned long)start + size + L1_CACHE_BYTES-1)
		& ~(L1_CACHE_BYTES-1);

	for (v = begin; v < end; v+=L1_CACHE_BYTES) {
		unsigned long p;

		p = __pa(v);

		for (j=0; j<CACHE_OC_NUM_WAYS; j++) {
			unsigned long data, addr;

			addr = CACHE_OC_ADDRESS_ARRAY|(j<<CACHE_OC_WAY_SHIFT)|
				(v&CACHE_OC_ENTRY_MASK);
			save_and_cli(flags);
			data = ctrl_inl(addr);
			if ( 	(data & CACHE_PHYSADDR_MASK) == (p &  CACHE_PHYSADDR_MASK)) {
				data &= ~CACHE_UPDATED;
				ctrl_outl(data, addr);
				restore_flags(flags);
				break;
			}
			restore_flags(flags);
		}
	}
}







/*
 * Write back the dirty D-caches and invalidate them.
 *
 * START: Virtual Address (U0, P1, or P3)
 * SIZE: Size of the region.
 */
inline void __flush_purge_region(void *start, int size)
{
	unsigned long v;
	unsigned long begin, end;

	begin = (unsigned long)start & ~(L1_CACHE_BYTES-1);
	end = ((unsigned long)start + size + L1_CACHE_BYTES-1)
		& ~(L1_CACHE_BYTES-1);

	for (v = begin; v < end; v+=L1_CACHE_BYTES) {
		unsigned long data, addr;

		data = (v & 0xfffffc00); /* _Virtual_ address, ~U, ~V */
		addr = CACHE_OC_ADDRESS_ARRAY | (v&CACHE_OC_ENTRY_MASK) |
			CACHE_OC_ASSOC_BIT;
		ctrl_outl(data, addr);
	}
}

/*
 * No write back please
 *
 * Except I don't think there's any way to avoid the writeback. So we
 * just alias it to __flush_purge_region(). dwmw2.
 */
void __flush_invalidate_region(void *start, int size)
	__attribute__((alias("__flush_purge_region")));





/* The 32KB cache on the SH7705 suffers from the same synonym problem
 * as SH4 CPUs */
#if defined(CONFIG_CPU_SUBTYPE_SH7705) && defined(CONFIG_SH_CACHE_32KB)






inline void __flush_purge_virtual_region(void *p1, void * virt, int size)
{
	unsigned long v, j;
	unsigned long begin, end;
	unsigned long p1_begin;


	begin = (unsigned long)virt & ~(L1_CACHE_BYTES-1);
	end = ((unsigned long)virt + size + L1_CACHE_BYTES-1)
		& ~(L1_CACHE_BYTES-1);

	p1_begin = (unsigned long)p1 & ~(L1_CACHE_BYTES - 1);
	/* do this the slow way as we may not have TLB entries
	 * for virt yet. */
	for (v = begin; v < end; v+=L1_CACHE_BYTES) {
		unsigned long p;

		p = __pa(p1_begin);

		for (j=0; j<CACHE_OC_NUM_WAYS; j++) {
			unsigned long data, addr;

			addr = CACHE_OC_ADDRESS_ARRAY|(j<<CACHE_OC_WAY_SHIFT)|
				(v&CACHE_OC_ENTRY_MASK);

			data = ctrl_inl(addr);
			if ( 	(data & CACHE_PHYSADDR_MASK) == (p &  CACHE_PHYSADDR_MASK)) {
				data &= ~(CACHE_UPDATED|CACHE_VALID);
				ctrl_outl(data, addr);
			}
		}
		p1_begin += L1_CACHE_BYTES;
	}
}

/*
 * Write back the range of D-cache, and purge the I-cache.
 *
 * Called from kernel/module.c:sys_init_module and routine for a.out format.
 */
void flush_icache_range(unsigned long start, unsigned long end)
{
	__flush_wback_region((void *)start, end - start);
}


/*
 * Writeback&Invalidate the D-cache of the page
 */
static void __flush_dcache_page(unsigned long phys)
{
	unsigned long addr, data, j;
	unsigned long flags;

	phys |= CACHE_VALID;

/* Here, phys is the physical address of the page. We check all the tags in
 * the cache for those with the same page number as this page (by masking off
 * the lowest 2 bits of the 19-bit tag; these bits are derived from the offset
 * within in the 4k page). Matching valid entries are invalidated.
 *
 * Since 2 bits of the cache index are derived from the virtual page number,
 * knowing this would reduce the number of cache entries to be searched by a
 * factor of 4. However this function exists to deal with potential cache
 * aliasing, therefore the optimisation is probably not possible.
 */
	save_and_cli(flags);
	jump_to_P2();

	/* In associative mode, we need to check both ways */
	for (j = 0; j < CACHE_OC_NUM_WAYS; j++) {

	        /* Loop all the D-cache */
                for (addr = CACHE_OC_ADDRESS_ARRAY | (j << CACHE_OC_WAY_SHIFT);
	             addr < ((CACHE_OC_ADDRESS_ARRAY | (j << CACHE_OC_WAY_SHIFT))
		             + (CACHE_OC_NUM_ENTRIES << CACHE_OC_ENTRY_SHIFT));
	             addr += (1 << CACHE_OC_ENTRY_SHIFT)) {
		        data = ctrl_inl(addr) & (0x1ffffC00 | CACHE_VALID);
		        if (data == phys) {
				data &= ~(CACHE_VALID | CACHE_UPDATED);
				ctrl_outl(data, addr);
			}
                }
	}

	back_to_P1();
	restore_flags(flags);
}


/*
 * Write back & invalidate the D-cache of the page.
 * (To avoid "alias" issues)
 */
void flush_dcache_page(struct page *page)
{
	if (test_bit(PG_mapped, &page->flags))
		__flush_dcache_page(PHYSADDR(page_address(page)));
}

void flush_cache_all(void)
{
	unsigned long flags;

	save_and_cli(flags);
	jump_to_P2();

	cache_wback_all();
	back_to_P1();
	restore_flags(flags);
}

void flush_cache_mm(struct mm_struct *mm)
{
	/* Is there any good way? */
	/* XXX: possibly call flush_cache_range for each vm area */
	flush_cache_all();
}

/*
 * Write back and invalidate D-caches.
 *
 * START, END: Virtual Address (U0 address)
 *
 * NOTE: We need to flush the _physical_ page entry.
 * Flushing the cache lines for U0 only isn't enough.
 * We need to flush for P1 too, which may contain aliases.
 */
void flush_cache_range(struct mm_struct *mm, unsigned long start,
		       unsigned long end)
{

	/*
	 * We could call flush_cache_page for the pages of these range,
	 * but it's not efficient (scan the caches all the time...).
	 *
	 * We can't use A-bit magic, as there's the case we don't have
	 * valid entry on TLB.
	 */
	flush_cache_all();
}

/*
 * Write back and invalidate I/D-caches for the page.
 *
 * ADDRESS: Virtual Address (U0 address)
 */
void flush_cache_page(struct vm_area_struct *vma, unsigned long address)
{
	pgd_t *dir;
	pmd_t *pmd;
	pte_t *pte;
	pte_t entry;
	unsigned long phys;

	dir = pgd_offset(vma->vm_mm, address);
	pmd = pmd_offset(dir, address);
	if (pmd_none(*pmd) || pmd_bad(*pmd))
		return;
	pte = pte_offset(pmd, address);
	entry = *pte;
	if (pte_none(entry) || !pte_present(entry))
		return;

	phys = pte_val(entry)&PTE_PHYS_MASK;
	__flush_dcache_page(phys);
}

/*
 * This is called when a page-cache page is about to be mapped into a
 * user process' address space.  It offers an opportunity for a
 * port to ensure d-cache/i-cache coherency if necessary.
 *
 * Not entirely sure why this is necessary on SH3 with 32K cache but
 * without it we get occasional "Memory fault" when loading a program.
 */
void flush_icache_page(struct vm_area_struct *vma, struct page *page)
{
	__flush_purge_region(page_address(page), PAGE_SIZE);
}

/*
 * clear_user_page
 * @to: P1 address
 * @address: U0 address to be mapped
 */
void clear_user_page(void *to, unsigned long address)
{
	struct page *page = virt_to_page(to);

	__set_bit(PG_mapped, &page->flags);
	if ((( address ^ (unsigned long)to) & CACHE_ALIAS) == 0){
		clear_page(to);
		__flush_wback_region(to, PAGE_SIZE);
	}
	else {
		__flush_purge_virtual_region(to,
					     (void *)(address & 0xfffff000), 
					     PAGE_SIZE);
		clear_page(to);
		__flush_wback_region(to, PAGE_SIZE);
	}
}

/*
 * copy_user_page
 * @to: P1 address
 * @from: P1 address
 * @address: U0 address to be mapped
 */
void copy_user_page(void *to, void *from, unsigned long address)
{
	struct page *page = virt_to_page(to);


	__set_bit(PG_mapped, &page->flags);
	if (((address ^ (unsigned long)to) & CACHE_ALIAS) == 0) {
		copy_page(to, from);
		__flush_wback_region(to, PAGE_SIZE);
	}
	else {
		__flush_purge_virtual_region(to, (void *)(address & 0xfffff000), PAGE_SIZE);
		copy_page(to, from);
		__flush_wback_region(to, PAGE_SIZE);
	}
}

#endif

